import _isEqual from 'lodash/isEqual';
import _max from 'lodash/max';
import _min from 'lodash/min';

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

/*
 * This file is part of the nivo project.
 *
 * Copyright 2016-present, Raphaël Benitte.
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */
import React from 'react';
import PropTypes from 'prop-types';

import { TransitionMotion, spring } from 'react-motion';
import compose from 'recompose/compose';
import defaultProps from 'recompose/defaultProps';
import withPropsOnChange from 'recompose/withPropsOnChange';
import pure from 'recompose/pure';
import { scaleOrdinal, scaleLinear } from 'd3-scale';
import { withTheme, withDimensions, withMotion } from '../../../hocs';
import { getInheritedColorGenerator, colorMotionSpring, getInterpolatedColor, guessQuantizeColorScale } from '../../../lib/colors';
import { getAccessorFor } from '../../../lib/propertiesConverters';
import { quantizeColorScalePropType } from '../../../props';
import Container from '../Container';
import SvgWrapper from '../SvgWrapper';
import Grid from '../../axes/Grid';
import Axes from '../../axes/Axes';
import HeatMapCellRect from './HeatMapCellRect';
import HeatMapCellCircle from './HeatMapCellCircle';

var computeX = function computeX(column, cellWidth, padding) {
    return column * cellWidth + cellWidth * 0.5 + padding * column + padding;
};
var computeY = function computeY(row, cellHeight, padding) {
    return row * cellHeight + cellHeight * 0.5 + padding * row + padding;
};

var HeatMap = function HeatMap(_ref) {
    var data = _ref.data,
        getIndex = _ref.getIndex,
        keys = _ref.keys,
        cellWidth = _ref.cellWidth,
        cellHeight = _ref.cellHeight,
        sizeScale = _ref.sizeScale,
        xScale = _ref.xScale,
        yScale = _ref.yScale,
        offsetX = _ref.offsetX,
        offsetY = _ref.offsetY,
        margin = _ref.margin,
        width = _ref.width,
        height = _ref.height,
        outerWidth = _ref.outerWidth,
        outerHeight = _ref.outerHeight,
        cellShape = _ref.cellShape,
        cellOpacity = _ref.cellOpacity,
        cellBorderWidth = _ref.cellBorderWidth,
        getCellBorderColor = _ref.getCellBorderColor,
        axisTop = _ref.axisTop,
        axisRight = _ref.axisRight,
        axisBottom = _ref.axisBottom,
        axisLeft = _ref.axisLeft,
        enableGridX = _ref.enableGridX,
        enableGridY = _ref.enableGridY,
        getLabelTextColor = _ref.getLabelTextColor,
        theme = _ref.theme,
        colorScale = _ref.colorScale,
        animate = _ref.animate,
        motionStiffness = _ref.motionStiffness,
        motionDamping = _ref.motionDamping,
        isInteractive = _ref.isInteractive;

    var Cell = void 0;
    if (cellShape === 'rect') {
        Cell = HeatMapCellRect;
    } else if (cellShape === 'circle') {
        Cell = HeatMapCellCircle;
    } else {
        Cell = cellShape;
    }

    var nodes = data.reduce(function (acc, d) {
        keys.forEach(function (key) {
            acc.push({
                key: key + '.' + getIndex(d),
                xKey: key,
                yKey: getIndex(d),
                x: xScale(key),
                y: yScale(getIndex(d)),
                width: sizeScale ? Math.min(sizeScale(d[key]) * cellWidth, cellWidth) : cellWidth,
                height: sizeScale ? Math.min(sizeScale(d[key]) * cellHeight, cellHeight) : cellHeight,
                value: d[key],
                color: colorScale(d[key])
            });
        });

        return acc;
    }, []);

    var motionProps = {
        animate: animate,
        motionDamping: motionDamping,
        motionStiffness: motionStiffness
    };

    var cells = void 0;
    if (animate === true) {
        cells = React.createElement(
            TransitionMotion,
            {
                styles: nodes.map(function (node) {
                    return {
                        key: node.key,
                        data: node,
                        style: _extends({
                            x: spring(node.x, motionProps),
                            y: spring(node.y, motionProps),
                            width: spring(node.width, motionProps),
                            height: spring(node.height, motionProps)
                        }, colorMotionSpring(node.color, motionProps))
                    };
                })
            },
            function (interpolatedStyles) {
                return React.createElement(
                    'g',
                    null,
                    interpolatedStyles.map(function (_ref2) {
                        var key = _ref2.key,
                            style = _ref2.style,
                            node = _ref2.data;

                        var color = getInterpolatedColor(style);

                        return React.createElement(Cell, {
                            key: key,
                            value: node.value,
                            x: style.x,
                            y: style.y,
                            width: Math.max(style.width, 0),
                            height: Math.max(style.height, 0),
                            color: color,
                            opacity: cellOpacity,
                            borderWidth: cellBorderWidth,
                            borderColor: getCellBorderColor(_extends({}, node, { color: color })),
                            textColor: getLabelTextColor(_extends({}, node, { color: color }))
                        });
                    })
                );
            }
        );
    } else {
        cells = nodes.map(function (node) {
            return React.createElement(Cell, {
                key: node.key,
                value: node.value,
                x: node.x,
                y: node.y,
                width: node.width,
                height: node.height,
                color: node.color,
                opacity: cellOpacity,
                borderWidth: cellBorderWidth,
                borderColor: getCellBorderColor(node),
                textColor: getLabelTextColor(node)
            });
        });
    }

    return React.createElement(
        Container,
        { isInteractive: isInteractive, theme: theme },
        function (_ref3) {
            var showTooltip = _ref3.showTooltip,
                hideTooltip = _ref3.hideTooltip;

            return React.createElement(
                SvgWrapper,
                {
                    width: outerWidth,
                    height: outerHeight,
                    margin: Object.assign({}, margin, {
                        top: margin.top + offsetY,
                        left: margin.left + offsetX
                    })
                },
                React.createElement(Grid, _extends({
                    theme: theme,
                    width: width - offsetX * 2,
                    height: height - offsetY * 2,
                    xScale: enableGridX ? xScale : null,
                    yScale: enableGridY ? yScale : null
                }, motionProps)),
                React.createElement(Axes, _extends({
                    xScale: xScale,
                    yScale: yScale,
                    width: width,
                    height: height,
                    theme: theme,
                    top: axisTop,
                    right: axisRight,
                    bottom: axisBottom,
                    left: axisLeft
                }, motionProps)),
                cells
            );
        }
    );
};

HeatMap.propTypes = {
    // data
    data: PropTypes.arrayOf(PropTypes.object).isRequired,
    indexBy: PropTypes.oneOfType([PropTypes.string, PropTypes.func]).isRequired,
    getIndex: PropTypes.func.isRequired, // computed
    keys: PropTypes.arrayOf(PropTypes.string).isRequired,

    minValue: PropTypes.oneOfType([PropTypes.oneOf(['auto']), PropTypes.number]).isRequired,
    maxValue: PropTypes.oneOfType([PropTypes.oneOf(['auto']), PropTypes.number]).isRequired,

    forceSquare: PropTypes.bool.isRequired,
    sizeVariation: PropTypes.number.isRequired,
    padding: PropTypes.number.isRequired,

    // cells
    cellShape: PropTypes.oneOfType([PropTypes.oneOf(['rect', 'circle']), PropTypes.func]).isRequired,
    cellOpacity: PropTypes.number.isRequired,
    cellHoverOpacity: PropTypes.number.isRequired,
    cellBorderWidth: PropTypes.number.isRequired,
    cellBorderColor: PropTypes.oneOfType([PropTypes.string, PropTypes.func]).isRequired,
    getCellBorderColor: PropTypes.func.isRequired, // computed

    // axes & grid
    axisTop: PropTypes.object,
    axisRight: PropTypes.object,
    axisBottom: PropTypes.object,
    axisLeft: PropTypes.object,
    enableGridX: PropTypes.bool.isRequired,
    enableGridY: PropTypes.bool.isRequired,

    // labels
    enableLabels: PropTypes.bool.isRequired,
    labelTextColor: PropTypes.oneOfType([PropTypes.string, PropTypes.func]).isRequired,
    getLabelTextColor: PropTypes.func.isRequired, // computed

    // theming
    colors: quantizeColorScalePropType.isRequired,
    colorScale: PropTypes.func.isRequired, // computed

    // interactivityP
    isInteractive: PropTypes.bool
};

export var HeatMapDefaultProps = {
    indexBy: 'id',

    minValue: 'auto',
    maxValue: 'auto',

    forceSquare: false,
    sizeVariation: 0,
    padding: 0,

    // cells
    cellShape: 'rect',
    cellOpacity: 0.85,
    cellHoverOpacity: 1,
    cellBorderWidth: 0,
    cellBorderColor: 'inherit',

    // axes & grid
    axisTop: {},
    axisLeft: {},
    enableGridX: false,
    enableGridY: false,

    // labels
    enableLabels: true,
    labelTextColor: 'inherit:darker(1.4)',

    // theming
    colors: 'nivo',

    // interactivity
    isInteractive: true
};

HeatMap.defaultProps = HeatMapDefaultProps;

var enhance = compose(defaultProps(HeatMapDefaultProps), withTheme(), withDimensions(), withMotion(), withPropsOnChange(['colors'], function (_ref4) {
    var colors = _ref4.colors;
    return {
        colorScale: guessQuantizeColorScale(colors)
    };
}), withPropsOnChange(['indexBy'], function (_ref5) {
    var indexBy = _ref5.indexBy;
    return {
        getIndex: getAccessorFor(indexBy)
    };
}), withPropsOnChange(['data', 'keys', 'width', 'height', 'padding', 'forceSquare'], function (_ref6) {
    var data = _ref6.data,
        keys = _ref6.keys,
        width = _ref6.width,
        height = _ref6.height,
        padding = _ref6.padding,
        forceSquare = _ref6.forceSquare;

    var columns = keys.length;
    var rows = data.length;

    var cellWidth = Math.max((width - padding * (columns + 1)) / columns, 0);
    var cellHeight = Math.max((height - padding * (rows + 1)) / rows, 0);

    var offsetX = 0;
    var offsetY = 0;
    if (forceSquare === true) {
        var cellSize = Math.min(cellWidth, cellHeight);
        cellWidth = cellSize;
        cellHeight = cellSize;

        offsetX = (width - ((cellWidth + padding) * columns + padding)) / 2;
        offsetY = (height - ((cellHeight + padding) * rows + padding)) / 2;
    }

    return {
        cellWidth: cellWidth,
        cellHeight: cellHeight,
        offsetX: offsetX,
        offsetY: offsetY
    };
}), withPropsOnChange(['data', 'getIndex'], function (_ref7) {
    var data = _ref7.data,
        getIndex = _ref7.getIndex;
    return {
        indices: data.map(getIndex)
    };
}), withPropsOnChange(function (prev, next) {
    return prev.keys !== next.keys || prev.cellWidth !== next.cellWidth || prev.cellHeight !== next.cellHeight || prev.padding !== next.padding || !_isEqual(prev.indices, next.indices);
}, function (_ref8) {
    var indices = _ref8.indices,
        keys = _ref8.keys,
        cellWidth = _ref8.cellWidth,
        cellHeight = _ref8.cellHeight,
        padding = _ref8.padding;
    return {
        xScale: scaleOrdinal(keys.map(function (key, i) {
            return computeX(i, cellWidth, padding);
        })).domain(keys),
        yScale: scaleOrdinal(indices.map(function (d, i) {
            return computeY(i, cellHeight, padding);
        })).domain(indices)
    };
}), withPropsOnChange(['data', 'keys', 'minValue', 'maxValue'], function (_ref9) {
    var data = _ref9.data,
        keys = _ref9.keys,
        _minValue = _ref9.minValue,
        _maxValue = _ref9.maxValue;

    var minValue = _minValue;
    var maxValue = _maxValue;
    if (minValue === 'auto' || maxValue === 'auto') {
        var allValues = data.reduce(function (acc, row) {
            return acc.concat(keys.map(function (key) {
                return row[key];
            }));
        }, []);

        if (minValue === 'auto') minValue = _min(allValues);
        if (maxValue === 'auto') maxValue = _max(allValues);
    }

    return {
        minValue: Math.min(minValue, maxValue),
        maxValue: Math.max(maxValue, minValue)
    };
}), withPropsOnChange(['colorScale', 'minValue', 'maxValue'], function (_ref10) {
    var colorScale = _ref10.colorScale,
        minValue = _ref10.minValue,
        maxValue = _ref10.maxValue;
    return {
        colorScale: colorScale.domain([minValue, maxValue])
    };
}), withPropsOnChange(['sizeVariation', 'minValue', 'maxValue'], function (_ref11) {
    var sizeVariation = _ref11.sizeVariation,
        minValue = _ref11.minValue,
        maxValue = _ref11.maxValue;

    var sizeScale = void 0;
    if (sizeVariation > 0) {
        sizeScale = scaleLinear().range([1 - sizeVariation, 1]).domain([minValue, maxValue]);
    }

    return { sizeScale: sizeScale };
}), withPropsOnChange(['cellBorderColor'], function (_ref12) {
    var cellBorderColor = _ref12.cellBorderColor;
    return {
        getCellBorderColor: getInheritedColorGenerator(cellBorderColor)
    };
}), withPropsOnChange(['labelTextColor'], function (_ref13) {
    var labelTextColor = _ref13.labelTextColor;
    return {
        getLabelTextColor: getInheritedColorGenerator(labelTextColor)
    };
}), pure);

export default enhance(HeatMap);